@@ -1,6 +1,6 @@
 // Class experimental::filesystem::path -*- C++ -*-
 
-// Copyright (C) 2014-2018 Free Software Foundation, Inc.
+// Copyright (C) 2014-2019 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -61,6 +61,12 @@ path::replace_filename(const path& replacement)
   return *this;
 }
 
+#ifdef _GLIBCXX_FILESYSTEM_IS_WINDOWS
+const fs::path::value_type dot = L'.';
+#else
+const fs::path::value_type dot = '.';
+#endif
+
 path&
 path::replace_extension(const path& replacement)
 {
@@ -78,8 +84,8 @@ path::replace_extension(const path& replacement)
 	  _M_pathname.erase(back._M_pos + ext.second);
 	}
     }
-  if (!replacement.empty() && replacement.native()[0] != '.')
-    _M_pathname += '.';
+  if (!replacement.empty() && replacement.native()[0] != dot)
+    _M_pathname += dot;
   _M_pathname += replacement.native();
   _M_split_cmpts();
   return *this;
@@ -297,7 +303,7 @@ path::has_filename() const
 std::pair<const path::string_type*, std::size_t>
 path::_M_find_extension() const
 {
-  const std::string* s = nullptr;
+  const string_type* s = nullptr;
 
   if (_M_type != _Type::_Multi)
     s = &_M_pathname;
@@ -312,14 +318,14 @@ path::_M_find_extension() const
     {
       if (auto sz = s->size())
 	{
-	  if (sz <= 2 && (*s)[0] == '.')
+	  if (sz <= 2 && (*s)[0] == dot)
 	    {
-	      if (sz == 1 || (*s)[1] == '.')  // filename is "." or ".."
+	      if (sz == 1 || (*s)[1] == dot)  // filename is "." or ".."
 		return { s, string_type::npos };
 	      else
 		return { s, 0 };  // filename is like ".?"
 	    }
-	  return { s, s->rfind('.') };
+	  return { s, s->rfind(dot) };
 	}
     }
   return {};
@@ -334,28 +340,6 @@ path::_M_split_cmpts()
   if (_M_pathname.empty())
     return;
 
-  {
-    // Approximate count of components, to reserve space in _M_cmpts vector:
-    int count = 1;
-    bool saw_sep_last = _S_is_dir_sep(_M_pathname[0]);
-    bool saw_non_sep = !saw_sep_last;
-    for (value_type c : _M_pathname)
-      {
-	if (_S_is_dir_sep(c))
-	  saw_sep_last = true;
-	else if (saw_sep_last)
-	  {
-	    ++count;
-	    saw_sep_last = false;
-	    saw_non_sep = true;
-	  }
-      }
-    if (saw_non_sep && saw_sep_last)
-      ++count; // empty filename after trailing slash
-    if (count > 1)
-      _M_cmpts.reserve(count);
-  }
-
   size_t pos = 0;
   const size_t len = _M_pathname.size();
 
@@ -378,13 +362,9 @@ path::_M_split_cmpts()
 	      pos = 3;
 	      while (pos < len && !_S_is_dir_sep(_M_pathname[pos]))
 		++pos;
-	      if (pos == len)
-		{
-		  _M_type = _Type::_Root_name;
-		  return;
-		}
 	      _M_add_root_name(pos);
-	      _M_add_root_dir(pos);
+	      if (pos < len) // also got root directory
+		_M_add_root_dir(pos);
 	    }
 	  else
 	    {
@@ -393,11 +373,6 @@ path::_M_split_cmpts()
 	      _M_add_root_dir(0);
 	    }
 	}
-      else if (len == 1) // got root directory only
-	{
-	  _M_type = _Type::_Root_dir;
-	  return;
-	}
       else // got root directory
 	_M_add_root_dir(0);
       ++pos;
@@ -406,28 +381,12 @@ path::_M_split_cmpts()
   else if (len > 1 && _M_pathname[1] == L':')
     {
       // got disk designator
-      if (len == 2)
-	{
-	  _M_type = _Type::_Root_name;
-	  return;
-	}
       _M_add_root_name(2);
       if (len > 2 && _S_is_dir_sep(_M_pathname[2]))
 	_M_add_root_dir(2);
       pos = 2;
     }
 #endif
-  else
-    {
-      size_t n = 1;
-      for (; n < _M_pathname.size() && !_S_is_dir_sep(_M_pathname[n]); ++n)
-	{ }
-      if (n == _M_pathname.size())
-	{
-	  _M_type = _Type::_Filename;
-	  return;
-	}
-    }
 
   size_t back = pos;
   while (pos < len)
@@ -452,7 +411,7 @@ path::_M_split_cmpts()
 	{
 	  const auto& last = _M_cmpts.back();
 	  pos = last._M_pos + last._M_pathname.size();
-	  _M_cmpts.emplace_back(string_type(1, '.'), _Type::_Filename, pos);
+	  _M_cmpts.emplace_back(string_type(1, dot), _Type::_Filename, pos);
 	}
     }
 
@@ -526,28 +485,34 @@ fs::hash_value(const path& p) noexcept
   return seed;
 }
 
-namespace std
+#include <experimental/string_view>
+
+std::string
+fs::filesystem_error::_M_gen_what()
 {
-_GLIBCXX_BEGIN_NAMESPACE_VERSION
-namespace filesystem
-{
-  extern string
-  fs_err_concat(const string& __what, const string& __path1,
-		const string& __path2);
-} // namespace filesystem
-
-namespace experimental::filesystem::v1 {
-_GLIBCXX_BEGIN_NAMESPACE_CXX11
-
-  std::string filesystem_error::_M_gen_what()
-  {
-    using std::filesystem::fs_err_concat;
-    return fs_err_concat(system_error::what(), _M_path1.native(),
-			 _M_path2.native());
-  }
-
-_GLIBCXX_END_NAMESPACE_CXX11
-} // namespace experimental::filesystem::v1
-
-_GLIBCXX_END_NAMESPACE_VERSION
-} // namespace std
+  const std::string pstr1 = _M_path1.u8string();
+  const std::string pstr2 = _M_path2.u8string();
+  experimental::string_view s = this->system_error::what();
+  const size_t len = 18 + s.length()
+    + (pstr1.length() || pstr2.length() ? pstr1.length() + 3 : 0)
+    + (pstr2.length() ? pstr2.length() + 3 : 0);
+  std::string w;
+  w.reserve(len);
+  w = "filesystem error: ";
+  w.append(s.data(), s.length());
+  if (!pstr1.empty())
+    {
+      w += " [";
+      w += pstr1;
+      w += ']';
+    }
+  if (!pstr2.empty())
+    {
+      if (pstr1.empty())
+	w += " []";
+      w += " [";
+      w += pstr2;
+      w += ']';
+    }
+  return w;
+}
